# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import base64
import hmac
import hashlib
import qrcode
from io import BytesIO


class wizard_banfin_qrcode(osv.TransientModel):
    """
        Classe de Wizard para gerar qr_code para autenticacao em 2 fatores
        prefixo: wiz_qc_
    """
    _name = 'wizard.banfin.qrcode'
    _columns = {
        'id': fields.integer('id'),
        'wiz_qc_user_id': fields.many2one('wizard.banfin.qrcode.users',
                                           'Usuario',
                                           ondelete="cascade",
                                           domain="[('wiz_qc_capa_id', '=', id)]",
                                           select=True,
                                           ),
        'state': fields.selection([('1', 'Iniciado'),
                                   ('2', 'Buscou usuarios'),
                                   ('3', 'Gerou qrCode')], "Estado", required=True, size=1),
        'wiz_qc_imagem': fields.binary('qrCode'),
        'wiz_qc_chave': fields.char('Chave', size=128),
    }
    _defaults = {
        'state': '1',
    }

    def banfin_wiz_btn_buscar_user(self, cr, uid, ids, context=None):
        grupo_id = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'bancos_financeiro', 'group_bancos_financeiro_manager')[1]
        usuario_ids = self.pool.get('res.users').search(cr, uid, [('groups_id', '=', grupo_id)])
        for brwUser in self.pool.get('res.users').browse(cr, uid, usuario_ids, context=context):
            vals = {'name': '%s-%s' % (brwUser.id, brwUser.name), 'wiz_qc_capa_id': ids[0]}
            self.pool.get('wizard.banfin.qrcode.users').create(cr, uid, vals, context=context)
        self.write(cr, uid, ids, {'state': '2'}, context=context)
        return True

    def banfin_wiz_btn_gerar(self, cr, uid, ids, context=None):
        brwUser = self.pool.get('res.users').browse(cr, uid, uid)
        company_id = brwUser.company_id.id
        usuario_id = self.browse(cr, uid, ids[0], context=context).wiz_qc_user_id.id
        key = self.pool.get('banfin.util').bfutl_gerar_chave(cr, uid, company_id, usuario_id, context=context)
        issuer_name = "SigERP"
        account_name = "%s-%s" % (brwUser.id, brwUser.name.split(' ')[0])
        otpauth_url = "otpauth://totp/%s?secret=%s&issuer=%s" % (account_name, key, issuer_name)
        img = qrcode.make(otpauth_url)
        buf = BytesIO()
        img.save(buf, format='PNG')
        imagem = buf.getvalue()
        imagem=base64.b64encode(imagem)
        self.write(cr, uid, ids, {'wiz_qc_imagem':imagem, 'state': '3', 'wiz_qc_chave': key}, context=context)
        return True


class wizard_banfin_qrcode_users(osv.TransientModel):
    """
        Classe de Wizard para gerar qr_code para autenticacao em 2 fatores
        prefixo: wiz_qc_
    """
    _name = 'wizard.banfin.qrcode.users'
    _columns = {
        'wiz_qc_capa_id': fields.many2one('wizard.banfin.qrcode', 'Capa', select=True, ondelete='cascade'),
        'name': fields.char('Nome', size=40),
    }
