# -*- coding: utf-8 -*-

from openerp.osv import osv, fields
from openerp.tools.translate import _


class wizard_acompanhamento_recebimento_registros(osv.TransientModel):
    _name = 'wizard.acompanhamento.recebimento.registros'
    _columns = {
        'recebimento_id': fields.many2one("wizard.acompanhamento.recebimento", 'Recebimento',
                                          ondelete="cascade", required=True, select=True),
        'coo': fields.integer('Coo'),
        'data': fields.date('Data'),
        'loja': fields.integer('Loja'),
        'caixa': fields.integer('Caixa'),
        'valor': fields.float('Valor', digits=(14,2)),
        'tipo': fields.selection([('1', _('Doacao')),
                                  ('2', _('Troco Digital'))], 'Tipo')
    }


class wizard_acompanhamento_recebimento(osv.TransientModel):
    _name = 'wizard.acompanhamento.recebimento'
    _columns = {
        'data_ini': fields.date('Data Inicial', required=True),
        'data_fim': fields.date('Data Final', required=True),
        'loja': fields.integer('Loja'),
        'caixa': fields.integer('Caixa'),
        'tipo': fields.selection([('1', 'Todos'),
                                  ('2', _('Doacao')),
                                  ('3', _('Troco Digital')),], 'Tipo'),
        'registro_ids': fields.one2many('wizard.acompanhamento.recebimento.registros', 'recebimento_id',
                                        'Recebimentos'),
    }

    _defaults = {
        'tipo': '1'
    }

    def button_pesquisar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        obj_parametros = self.pool.get('supervisor.cfg.parametros')
        dados = self.read(cr, uid, ids, [], context=context)
        datas = {}
        for x in dados:
            datas = {
                'data_ini': x.get('data_ini'),
                'data_fim': x.get('data_fim'),
                'loja': x.get('loja', False),
                'caixa': x.get('caixa', False),
                'tipo': x.get('tipo')
            }

        par_ids = obj_parametros.search(cr, uid, [], context=context)
        campos = ['supcfp_tot_doacao', 'supcfp_regex_tot_troco_digital']
        parametros = obj_parametros.read(cr, uid, par_ids[0], campos, context=context)
        tot = []
        if datas.get('tipo') == '1':
            tot.append(str(parametros.get('supcfp_tot_doacao')))
            tot.append(str(parametros.get('supcfp_regex_tot_troco_digital')))
        elif datas.get('tipo') == '2':
            tot.append(str(parametros.get('supcfp_tot_doacao')))
        elif datas.get('tipo') == '3':
            tot.append(str(parametros.get('supcfp_regex_tot_troco_digital')))

        tot = ','.join(tot)
        dados = self.get_dados(cr, datas, tot, context)
        if not dados:
            raise osv.except_osv(_('Atencao!'),
                                 _('Nao ha registros com estes filtros!\nVerifique os totalizadores de'
                                   ' doacao/troco digital no cadastro de parametros'))

        registros = []
        for i in dados:
            tot = i.pop('totalizador')
            if tot == parametros.get('supcfp_tot_doacao'):
                i['tipo'] = '1'
            elif tot == parametros.get('supcfp_regex_tot_troco_digital'):
                i['tipo'] = '2'

            registros.append([0, 0, i])

        obj_registros = self.pool.get('wizard.acompanhamento.recebimento.registros')
        registro_ids = obj_registros.search(cr, uid, [], context=context)
        if registro_ids:
            obj_registros.unlink(cr, uid, registro_ids, context=context)

        vals = {'registro_ids': registros}
        self.write(cr, uid, ids, vals, context)
        return True

    def button_print_recebimento(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        dados_filtros = self.read(cr, uid, ids[0], context=context)
        obj_registros = self.pool.get('wizard.acompanhamento.recebimento.registros')
        registro_ids = obj_registros.search(cr, uid, [('recebimento_id','in',ids)], context=context)
        dados_registros = obj_registros.read(cr, uid, registro_ids, context=context)
        datas= {'dados': dados_registros,
                'data_ini': dados_filtros.get('data_ini'),
                'data_fim': dados_filtros.get('data_fim'),
                'caixa': dados_filtros.get('caixa'),
                'loja': dados_filtros.get('loja')}
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'listagem_acompanhamento_recebimento',
            'datas': datas,
        }

    def get_dados(self, cr, datas, tot, context=None):
        context = context and context.copy() or {}
        caixa = datas.get('caixa')
        loja = datas.get('loja')
        data_ini = datas.get('data_ini')
        data_fim = datas.get('data_fim')
        cond_caixa = ''
        cond_loja = ''
        if caixa:
            cond_caixa = 'AND smc.supcap_caixa = %s' % caixa

        if loja:
            cond_loja = 'AND smc.supcap_loja = %s' % loja

        sql = """
                        SELECT smc.supcap_total_cupom        AS valor, 
                               smc.supcap_nro_cupom          AS coo,
                               smc.supcap_data_cupom         AS data,
                               smc.supcap_caixa              AS caixa,
                               smc.supcap_loja               AS loja,
                               smc.supcap_cod_tot_nao_fiscal AS totalizador
                        FROM supervisor_mov_capa  AS smc
                        WHERE smc.supcap_data_cupom >= '{data_ini}'
                          AND smc.supcap_data_cupom <= '{data_fim}'
                          AND smc.supcap_tipo = '{tipo}'
                          {cond_loja}
                          {cond_caixa}
                          AND smc.supcap_cod_tot_nao_fiscal in ({tot})
                        ORDER BY smc.supcap_loja, smc.supcap_caixa, smc.supcap_nro_cupom
                """.format(tipo='W',
                           data_ini=data_ini,
                           data_fim=data_fim,
                           cond_loja=cond_loja,
                           cond_caixa=cond_caixa,
                           tot=tot)
        cr.execute(sql)
        return cr.dictfetchall()

