# -*- coding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
# #############################################################################

from openerp.osv import osv
from SigUtils.ajustar_empresa import sigutils_conversar_com_a_sig
from openerp.modules.module import load_information_from_description_file as info
from SigUtils import sigerp, cnpj
from openerp.tools.translate import _
import logging
from openerp import SUPERUSER_ID
from ...web_sig.control.utils import websig_get_my_ip
from openerp.modules import get_module_resource
from openerp import tools
from datetime import datetime
from re import sub
import requests
import json

# Tipos de sistema
ERP = '1'
SUPERVISOR = '2'
ERP_SUPERVISOR = '3'
PDV = '4'
COMANDA = '5'
COLETOR = '6'

def log_debug(func):
    from openerp.tools import config as openerp_config
    from datetime import datetime
    import logging
    import time
    no_logger = ['pendencias']

    def wrapper(*args, **kwargs):
        if args[0]._name in no_logger and openerp_config.get('log_level').upper() == 'DEBUG':
            _logger = logging.getLogger(__name__)
            _logger.setLevel(logging.DEBUG)  # Enable DEBUG mode
        elif openerp_config.get('log_level').upper() == 'DEBUG':
            _logger = args[0]._logger
        else:
            return func(*args, **kwargs)
        start_time = time.time()
        result = func(*args, **kwargs)
        if _logger.isEnabledFor(logging.DEBUG):
            end_time = time.time()
            _logger.debug('%s :: %s@%s|%s (%.3fs)' % (
                func.__doc__ and func.__doc__.replace(
                    '\n', '', 1).strip().split('\n')[0] or '', func.func_name,
                datetime.fromtimestamp(start_time).strftime('%H:%M:%S'),
                datetime.fromtimestamp(end_time).strftime('%H:%M:%S'), end_time - start_time))

        return result

    return wrapper


class valida_pendencias(osv.Model):
    _inherit = 'valida.pendencias'
    _logger = logging.getLogger(_inherit)

    def valpen_verifica_versao(self, cr, uid, par=[], context={}):
        """Usado no frente via xmlrpc, o parametro nao eh usado, mas nao retirar pois vai dar erro no frente"""
        context = context and context.copy() or {}
        return '7'

    def _auto_init(self, cr, context=None):
        context = context and context.copy() or {}
        try:
            res = super(valida_pendencias, self)._auto_init(cr, context)
            _sql = "select name, latest_version, state from ir_module_module where state <> 'uninstalled' order by name"
            cr.execute(_sql)
            _dict_modulos = {'modulos': {}}
            for _module_dados in cr.dictfetchall():
                terp_file = get_module_resource(_module_dados.get('name'), '__openerp__.py')
                ver = ''
                try:
                    f = tools.file_open(terp_file)
                    ver = eval(f.read())['version']
                except:
                    pass
                finally:
                    f.close()
                _dict_modulos['modulos'][_module_dados.get('name')] = ver
            _xmodulos = str(_dict_modulos).replace("'", '"')
            vals = {'valpen_codigo': _xmodulos}
            pendencia_ids = self.search(cr, SUPERUSER_ID, [('valpen_codigo', '=', _xmodulos)],
                                        context=context)
            if not pendencia_ids:
                self.create(cr, 1, vals, context)
        except:
            pass

        return res

    def valpen_validar_erp(self, cr, uid, ids, context=None):
        """
        Método executado por heranca no modulo 'adm'
        @param cr:
        @param uid:
        @param ids:
        @param context:
        @return: True ou False dependendo se conseguiu ou não
        """
        context = context and context.copy() or {}
        return True

    def valpen_cron_enviar_versao (self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if self.valpen_verifica_rede_sig(cr, uid, context):
            return True

        vals = {}
        dados = []
        cnpj_cadastro = ''
        args = [('name', '=', 'supervisor'), ('state', 'in', ('installed', 'to upgrade'))]
        supervisor_instalado = self.pool.get('ir.module.module').search(cr, uid, args, context=context)
        erp = False
        args = [('name', '=', 'adm'), ('state', 'in', ('installed', 'to upgrade'))]
        if self.pool.get('ir.module.module').search(cr, uid, args, context=context):
            erp = True
            company_ids = self.pool.get('res.company').search(cr, uid, [])
            for _company_dados in self.pool.get('res.company').browse(cr, uid, company_ids, context=context):
                if _company_dados.rescon_pessoa_id.pessoa_tipo == 'F':
                    cnpj_cadastro = sub('[^0-9]', '', _company_dados.rescon_pessoa_id.cpf_cnpj)
                else:
                    res = cnpj.Cnpj(_company_dados.rescon_pessoa_id.cpf_cnpj).remove_edicao()
                    cnpj_cadastro = res.get('cnpj', '')
                if not cnpj_cadastro:
                    self._logger.info(_('Empresa(pessoa nro %s) sem cnpj cadastrado!') %
                                      _company_dados.rescon_pessoa_id.codigo)
                    continue

                vals = {'versao': info('adm').get('version'),
                        'cnpj': cnpj_cadastro,
                        'tipo': ERP}

                if supervisor_instalado:
                    vals['tipo'] = ERP_SUPERVISOR

                dados.append(vals)

        if supervisor_instalado:
            obj_config = self.pool.get('supervisor.cfg.configuracao')
            config_ids = obj_config.search(cr, uid, [], context=context)
            dados_conf = obj_config.read(cr, uid, config_ids[0], ['supcfc_cnpj'], context)
            cnpj_cadastro = sub(r'[^a-zA-Z0-9]', '', dados_conf.get('supcfc_cnpj') or '')
            if not cnpj_cadastro:
                self._logger.info(_('Empresa sem cnpj cadastrado!'))
                return True

            obj_pdv = self.pool.get('supervisor.cad.pdvs')
            pdv_ids = obj_pdv.search(cr, uid, [])
            for i in obj_pdv.read(cr, uid, pdv_ids, ['suppdv_versao', 'suppdv_caixa']):
                dados.append({'versao': i.get('suppdv_versao') or '',
                              'cnpj': cnpj_cadastro,
                              'tipo': PDV,
                              'caixa': i.get('suppdv_caixa')})
            if not erp:
                vals = {'versao': info('supervisor').get('version'),
                        'cnpj': cnpj_cadastro,
                        'tipo': SUPERVISOR}

                dados.append(vals)

        obj_dispositivo = self.pool.get('valida.dispositivo')
        dispositivo_ids = obj_dispositivo.search(cr, uid, [])
        campos =  ['valdis_tipo','valdis_versao', 'valdis_marca', 'valdis_modelo', 'valdis_nro_serie']
        for i in obj_dispositivo.read(cr, uid, dispositivo_ids, campos, context=context):
            tipo = {'1': COLETOR,
                    '2': COMANDA}.get(i.get('valdis_tipo'))
            dados.append({'versao': i.get('valdis_versao') or '',
                          'cnpj': cnpj_cadastro,
                          'tipo': tipo,
                          'marca': i.get('valdis_marca') or '',
                          'modelo': i.get('valdis_modelo') or '',
                          'serie': i.get('valdis_nro_serie') or ''})
        self.valpen_enviar_dados_versao(cr, uid, dados, context)
        return True

    def valpen_enviar_dados_versao(self, cr, uid, dados, context=None):
        context = context and context.copy() or {}
        obj_config_parameter = self.pool.get('ir.config_parameter')
        servidor = obj_config_parameter.get_sig_server(cr, uid, context=context).split(',')
        try:
            host = servidor[0]
            db = servidor[1]
            porta = servidor[2]
            user = servidor[3]
            senha = servidor[4]
            res = requests.post('https://%s/contrecf/set_versao_sistemas' % host,
                                auth=(user, senha),
                                data={'db': db,
                                      'pwd': senha,
                                      'user': user,
                                      'url': host,
                                      'port': porta,
                                      'dados': json.dumps(dados)
                                      },
                                timeout=10000)
            res = json.loads(res.content)
            if res.get('codigo') != '00':
                self._logger.error(res.get('mensagem'))
                return True

        except Exception as e:
                self._logger.error(_('Erro comunicando com servidor da SIG!:\n%s') % e)

        return True

    def valpen_ajustar_pendencias(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        self._logger.info(_('Ajustar pendencias! %s' % context))

        def _admpen_ajustar_versao_modulos():
            self._logger.info(_('Ajustando registro das versoes dos modulos'))
            res = False
            _company_dados = self.pool.get('res.company').browse(cr, uid, 1, context=context)
            try:
                _cnpj_cliente = _company_dados.rescon_pessoa_id.cpf_cnpj
                _nome_cliente = _company_dados.rescon_pessoa_id.nome
            except:
                _cnpj_cliente = ''
                _nome_cliente = ''
            try:
                _vals = eval(brw_pendencias.valpen_codigo)
                _vals['cnpj_cliente'] = _cnpj_cliente
                _vals['nome_cliente'] = _nome_cliente
                _vals['data'] = brw_pendencias.create_date
                servidor = self.pool.get('ir.config_parameter').get_sig_server(cr, uid).split(',')
                self._logger.info(
                    _('Conversando com a Sig(modulos): Host:%s, Porta:%s, User:%s, db:%s, context:%s') % (
                        servidor[0], servidor[2], servidor[3], servidor[1], context))
                _vals['ip_cliente'] = websig_get_my_ip()


                servidor = self.pool.get('ir.config_parameter').get_sig_server(cr, uid).split(',')
                servidor_local_id = self.pool.get('ir.config_parameter').search(cr, uid,
                                                                               [('key', '=','local.server.sig')])
                if servidor_local_id:
                    servidor_local = self.pool.get('ir.config_parameter').read(cr, uid, servidor_local_id, [])
                    servidor_local = servidor_local[0]['value']
                else:
                    servidor_local = 'iporanga'
                    self.pool.get('ir.config_parameter').create(cr, uid, {'key': 'local.server.sig',
                                                                          'value': servidor_local})
                res = sigutils_conversar_com_a_sig(servidor, 'sig.cliente.modulo.versao',
                                                   'sigclv_recebe_versao', _vals, 'S', servidor_local)
                self._logger.info(_('Fim de conversacao! Resultado: %s') % str(res))
            except Exception, e:
                self._logger.warn(_('Erro ao tentar conversar com a Sig! Erro: %s') % str(e))
                # Não conseguindo a conecção, remover o servidor da configuração para que possar ser gerado novamente,
                # talvez ele esteja errado
                server_id = self.pool.get('ir.config_parameter').search(cr, SUPERUSER_ID,
                                                                        [('key', '=', 'server_sig')], context=context)
                self.pool.get('ir.config_parameter').unlink(cr, SUPERUSER_ID, server_id)
            self._logger.info(_('Fim de registro de versoes!'))
            return res

        pendencias_ids = self.search(cr, uid, [], context=context)
        if pendencias_ids:
            for brw_pendencias in self.browse(cr, uid, pendencias_ids, context=context):
                if 'modulos' in brw_pendencias.valpen_codigo:
                    res = _admpen_ajustar_versao_modulos()
                    if res:
                        self.unlink(cr, uid, brw_pendencias.id, context=context)

        self._logger.info(_('Pendencias ajustadas'))
        return True

    @log_debug
    def valpen_verifica_rede_sig(self, cr, uid, context=None):
        """
        Verifica se o sistema esta rodando dentro da rede da sig
        :param cr:
        :param uid:
        :param context:
        :return: True ou False
        """
        context = context and context.copy() or {}
        server = self.pool.get('ir.config_parameter').get_sig_server(cr, uid,
                                                                     context=context).split(',')
        host = self.pool.get('ir.config_parameter').get_param(cr, uid, 'local.server.sig')
        try:
            port = server[2]
            usuario = server[3]
            senha = server[4]
        except:
            raise Exception('Erro no parametro "server_sig" do sistema, parametro incorreto')

        try:
            res = requests.post('http://%s:%s/valida/controller_verificar_rede_sig' % (host, port),
                                auth=(usuario, senha),
                                data={},
                                timeout=1000)
            res = json.loads(res.content)
            if res.get('codigo') != '00':
                self._logger.error(_('Erro ao verificar rede SIG: %s') % res.get('mensagem'))
                return False

        except Exception as e:
            return False

        return True

    @log_debug
    def valpen_verificar_licenca_ativa(self, cr, uid, context=None):
        '''
        Verifica se a licenca do sistema esta valida, em caso de negativa, dispara um raise erro
        :param cr:
        :param uid:
        :param context:
        :return:
        '''
        context = context and context.copy() or {}
        return True
        # Segunda-feira (Monday) e o primeiro dia, representado pelo numero 0.
        # para finais de semana, nao faz a verificacao
        if datetime.now().weekday() in [5,6]:
            return True

        licenca = self.pool.get('ir.config_parameter').get_param(cr, uid, 'licensa_sig_erp')
        if self.valpen_verifica_rede_sig(cr, uid, context):
            return True

        if not licenca:
            raise osv.except_osv(_('Atencao!'),
                                 _('Sistema sem licenca cadastrada!'))

        try:
            licenca = sigerp._crypto_sig('d', licenca)
        except Exception as e:
            self._logger.info(_('Erro ao descriptografar licenca: %s') % e)
            raise osv.except_osv(_('Atencao!'), _('Criptografia incorreta!'))

        list_licenca = licenca.split(';')
        if len(list_licenca) > 1:
            _dt = list_licenca[1]
            if _dt == '0':
                raise osv.except_osv(_('Atencao!'),
                                     _('Erro 9000 - Favor entrar em contato com o Suporte!'))

            _hoje = datetime.strftime(datetime.now(), '%Y%m%d')
            if _hoje > _dt:
                raise osv.except_osv(_('Atencao!'),
                                     _('Erro 9000 - Favor entrar em contato com o Suporte!'))

    def valpen_ativar_licenca(self, cr, uid, cripto_licenca, context=None):
        context = context and context.copy() or {}
        try:
            licenca = sigerp._crypto_sig('d', cripto_licenca)
        except Exception as e:
            self._logger.info(_('Erro ao descriptografar licenca: %s') % e)
            raise osv.except_osv(_('Atencao!'), _('Criptografia incorreta!'))

        list_licenca = licenca.split(';')
        cnpj = list_licenca[0]
        _company_dados = self.pool.get('res.company').browse(cr, uid, 1, context=context)
        try:
            cnpj_cadastro = sub(r'[^a-zA-Z0-9]', '', _company_dados.rescon_pessoa_id.cpf_cnpj)
        except:
            obj_config = self.pool.get('supervisor.cfg.configuracao')
            config_ids = obj_config.search(cr, uid, [], context=context)
            dados_conf = obj_config.read(cr, uid, config_ids[0], ['supcfc_cnpj'], context)
            cnpj_cadastro = sub(r'[^a-zA-Z0-9]', '', dados_conf.get('supcfc_cnpj') or '')

        if cnpj != cnpj_cadastro:
            raise osv.except_osv(_('Atencao!'),
                                 _('Cnpj da licenca diferente do cadastrado na empresa!'
                                   '\nCnpj Cadastrado: %s\nCnpj Licenca: %s' % (cnpj_cadastro, cnpj)))

        self.pool.get('ir.config_parameter').set_param(cr, uid, 'licensa_sig_erp', cripto_licenca)
        self.valpen_verificar_licenca_ativa(cr, uid, context)
        return True
